<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use \Teknisa\GER\Utilities\Messages;

class Fai00300GrupoServicoFaturamento extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $environment;
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $dataSourceManager;
    protected $parameterBagColumns;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $dataSourceManager, Messages $messages, $entityManager) {

        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->dataSourceManager  = $dataSourceManager;
        $this->messages = $messages;
        $this->entityManager = $entityManager;
        $this->connection = $this->entityManager->getConnection();
    }

    public function filtroGrupoServicoFaturamento(DTO\Request\Filter $request, DTO\Response $response) {

        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->addCondition('CDFILIAL', "=", $this->environment->getCdFilial());
        $this->dataSourceName = "grupservfat";
        $this->parameterBagColumns = array("CDFILIAL","CDCLIENTE");

        parent::find($request, $response);

    }

    public function salvaGrupoServico(DTO\Request\Row $request, DTO\Response $response){
        $this->dataSourceName = "fai00300_grupservfatfai";

        $row = $request->getRow();
        $row['CDSERAJEFEGSF'] = null;
        $row['QTEFMINDOMGSF'] = null; 
        $row['QTEFMINSEGGSF'] = null; 
        $row['QTEFMINTERGSF'] = null; 
        $row['QTEFMINQUAGSF'] = null; 
        $row['QTEFMINQUIGSF'] = null; 
        $row['QTEFMINSEXGSF'] = null; 
        $row['QTEFMINSABGSF'] = null; 
        $row['QTEFMINFERGSF'] = null; 
        
        if(!isset($row['CDTIPOOPER'])) {
            $row['CDTIPOOPER'] = null;
            $row['IDENTRSAIDOP'] = null;
        }
        
       $conditions = array(
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $row["CDFILIAL"]
			),
			array(
				"columnName" => "CDCLIENTE",
				"operator" => "=",
				"value" => $row["CDCLIENTE"]
			),
			array(
				"columnName" => "NRORG",
				"operator" => "=",
				"value" => $this->environment->getNrOrgTrab()
			),
			array(
			    "columnName" => "CDGRSERVFAT",
				"operator" => "=",
				"value" => $row["CDGRSERVFAT"]
            )
		);

		$filterCriteria = new FilterCriteria($this->dataSourceName, $conditions);
        $errorMessage = "GROUP_ALREADY_REGISTERED";

        $this->parameterBagColumns = array("CDFILIAL", "CDCLIENTE", 'NRORG', "CDGRSERVFAT");
        $newFilter = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilter);
        $registrosEncontrados = $dataSet->getRows();
        if(empty($registrosEncontrados)){
			parent::save($request, $response);
        }else{
        	if($row['__is_new']) {
				$response->setError(new Error($errorMessage, 1));
        	}else{
	        	parent::save($request, $response);
        	}
        }
    }

    public function salvaServicoFaturamento(DTO\Request\Row $request, DTO\Response $response) {

		$this->dataSourceName = "servclie";
		$row = $request->getRow();
		$row['__is_new'] = false;

        parent::save($request, $response);
		$this->salvaIdCorrEfet($row);
		
    }
    
    public function salvaIdCorrEfet($row){
        if($row['IDCORREFET'] == 'S'){
        	$params = array(
        		"P_CDSERVICO" => $row['CDSERVFILI'],
        		"P_CDFILIAL" => $row['CDFILIAL'],
        		"P_CDCLIENTE" => $row['CDCLIENTE'],
        		"P_CDGRUPO" => $row['CDGRSERVFAT']
        	);
        	
        	$query = "UPDATE 
    					  GRUPSERVFAT
    					SET 
    					  CDSERAJEFEGSF = :P_CDSERVICO
    					WHERE
    					  (CDFILIAL = :P_CDFILIAL)  AND
    					  (CDCLIENTE = :P_CDCLIENTE)  AND
    					  (CDGRSERVFAT = :P_CDGRUPO)";
        	
        	try {
                $std = $this-> connection->prepare($query);
                $std->execute($params);
                
            } catch (Exception $e) {
                throw $e;
            }
        }
    }
    
    public function getServicoCorrigeEfetivo(DTO\Request\Row $request, DTO\Response $response){
    	$row = $request->getRow();
    	$params = array(
    		"P_CDFILIAL" => $row['CDFILIAL'],
    		"P_CDCLIENTE" => $row['CDCLIENTE'],
    		"P_CDGRUPO" => $row['CDGRSERVFAT']
    	);
    	$query = "SELECT * 
					FROM GRUPSERVFAT
					WHERE
					  (CDFILIAL = :P_CDFILIAL)  AND
					  (CDCLIENTE = :P_CDCLIENTE)  AND
					  (CDGRSERVFAT = :P_CDGRUPO)";
    	
    	try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
            $retorno = $std->fetchAll();
            $response->addDataSet(new DataSet("DADOS", $retorno));
            
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function deleteServicoFaturamento(DTO\Request\Row $request, DTO\Response $response) {

		$this->dataSourceName = "servclie";
		$row = $request->getRow();
		$row['__is_new'] = false;
		$row['CDGRSERVFAT'] = null;

        parent::save($request, $response);

    }

    public function delete(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "fai00300_grupservfatfai";
        parent::delete($request, $response);
    }
    
    public function saveCorrecao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $sql = "INSERT INTO CORRPRECGRUP (
                    CDFILIAL, 
                    CDSERVFILI, 
                    CDCLIENTE, 
                    VRPEINICSERV, 
                    VRPEFIMSERV, 
                    VRPECORRSERV, 
                    QTINISERV, 
                    QTFINSERV, 
                    NRORG, 
                    DTINCLUSAO, 
                    DTULTATU, 
                    NRORGINCLUSAO, 
                    CDOPERINCLUSAO, 
                    NRORGULTATU, 
                    CDOPERULTATU, 
                    IDATIVO) VALUES (
                        :CDFILIAL,
                        :CDSERVFILI,
                        :CDCLIENTE,
                        :VRPEINICSERV,
                        :VRPEFIMSERV,
                        :VRPECORRSERV,
                        :QTINISERV,
                        :QTFINSERV,
                        :NRORG,
                        :DTINCLUSAO,
                        :DTULTATU,
                        :NRORGINCLUSAO,
                        :CDOPERINCLUSAO,
                        :NRORGULTATU,
                        :CDOPERULTATU,
                        :IDATIVO
                    )";
        $currentDate = $this->environment->getCurrentTime()->format('d/m/Y h:m:s');
        $params = array(
            'CDFILIAL'        => $row['CDFILIAL'],
            'CDSERVFILI'      => $row['CDSERVFILI'],
            'CDCLIENTE'       => $row['CDCLIENTE'],
            'VRPEINICSERV'    => $row['VRPEINICSERV'],
            'VRPEFIMSERV'     => $row['VRPEFIMSERV'],
            'VRPECORRSERV'    => $row['VRPECORRSERV'],
            'QTINISERV'       => $row['QTINISERV'],
            'QTFINSERV'       => $row['QTFINSERV'],
            'NRORG'           => $this->environment->getNrOrg(),
            'DTINCLUSAO'      => $currentDate,
            'DTULTATU'        => $currentDate,
            'NRORGINCLUSAO'   => $this->environment->getNrOrg(),
            'CDOPERINCLUSAO'  => $this->environment->getCdOperador(),
            'NRORGULTATU'     => $this->environment->getNrOrg(),
            'CDOPERULTATU'    => $this->environment->getCdOperador(),
            'IDATIVO'         => 'S'
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function removeCorrecao(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $sql = "DELETE FROM CORRPRECGRUP 
                    WHERE
                        CDFILIAL     = :CDFILIAL     AND
                        CDSERVFILI   = :CDSERVFILI   AND
                        CDCLIENTE    = :CDCLIENTE    AND
                        VRPEINICSERV = :VRPEINICSERV AND
                        NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        foreach($rows as $row){
            $params = array(
                'CDFILIAL'     => $row['CDFILIAL'],
                'CDSERVFILI'   => $row['CDSERVFILI'],
                'CDCLIENTE'    => $row['CDCLIENTE'],
                'VRPEINICSERV' => $row['VRPEINICSERV'],
                'NRORG'        => $row['NRORG']
            );
            $std->execute($params);
        }
    }
    
    public function editaCorrecao(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        try {
            $this->verifyInterval($rows);
        } catch (Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            return;
        } 
        foreach($rows as $row){
            $sql = "UPDATE CORRPRECSERV
                    SET VRPEINICSERV = :P_NEWVRPEINICSERV,
                        QTFINSERV = :P_QTFINSERV,
                        QTINISERV = :P_QTINISERV,
                        VRPECORRSERV = :P_VRPECORRSERV,
                        VRPEFIMSERV = :P_VRPEFIMSERV
                    WHERE
                        CDFILIAL = :P_CDFILIAL AND
                        CDSERVFILI = :P_CDSERVFILI AND
                        CDCLIENTE = :P_CDCLIENTE AND
                        VRPEINICSERV = :P_VRPEINICSERV AND
                        NRORG = :P_NRORG";
            
            $params = array(
                "P_CDFILIAL" => $row['CDFILIAL'],    
                "P_CDSERVFILI" => $row['CDSERVFILI'],    
                "P_CDCLIENTE" => $row['CDCLIENTE'],
                "P_VRPEINICSERV" => $row['VRPEINICSERV'],            
                "P_NRORG" => $row['NRORG'],
                "P_NEWVRPEINICSERV" => $row['NEWVRPEINICSERV'],
                "P_QTFINSERV" => $row['QTFINSERV'],
                "P_QTINISERV" => $row['QTINISERV'],
                "P_VRPECORRSERV" => $row['VRPECORRSERV'],
                "P_VRPEFIMSERV" => $row['VRPEFIMSERV']
            );
            try {
                $this->connection->beginTransaction();
                $std = $this->connection->prepare($sql);
                $std->execute($params);
                $this->connection->commit();
            } catch (Exception $e) {
                $this->connection->rollback();
                throw $e;
            }
        }
    }

    public function deleteGrupServCheckedRows(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $query = "UPDATE SERVCLIE SET CDGRSERVFAT = NULL WHERE CDFILIAL = :P_CDFILIAL AND CDCLIENTE = :P_CDCLIENTE AND CDGRSERVFAT = :P_CDGRUPO";
        $std = $this->connection->prepare($query);
        
        foreach ($row as $item) {
            $params = array(
                "P_CDFILIAL"  => $item['CDFILIAL'],
                "P_CDCLIENTE" => $item['CDCLIENTE'],
                "P_CDGRUPO"   => $item['CDGRSERVFAT']
            );
            $std->execute($params);
        }
    }
}